import numpy as np
import math
import scipy as sp
import matplotlib
from matplotlib import pyplot as plt
from scipy.interpolate import interp1d
import math
plt.rcParams.update({'font.size': 24,'legend.fontsize': 20,'legend.handlelength': 0.5})

# This script reads the QE output (.xml) and corrections.x output files (.dat) and generates Koopmans' plot
# To modify the script for different materials, change:
# prefix = 'example_MgO'
# Elat =  0.793486  should be read from corrections.x output file by grep Ecorr

#Constants
hartree2eV = 27.211396132
bohr2A = 0.529177249
A2bohr = 1.8897259886


def grep_ks_level(filename,charge,spin):
	if charge == 1 :    #+1
		E=[]
		occ=[]
		E_VBM=[]
		E_CBM=[]
		E_D_up=[]
		E_D_dn=[]
		f = open(filename,'r')
		data = f.readlines()
		for i,line in enumerate(data):
			s = line.split()
			if s[0][0:5] == "<nks>":
				nkpt = int(s[0][5])
			if s[0] == "<eigenvalues":
				nsize=int(s[1][6:9])
				#print('nsize='+str(nsize))
				for line2 in data[i+1:int(i+2+math.ceil(nsize/5))-1]:
					s2 = line2.split()
					for w in s2:
						E.append(float(w))
			if s[0] == "<occupations":
				for line2 in data[i+1:int(i+2+math.ceil(nsize/5))-1]:
					s2 = line2.split()
					for w in s2:
						occ.append(float(w))
		f.close()
		E=hartree2eV*np.array(E)
		occ=np.array(occ)
		if spin == 1: 
			band_VBM=int(np.sum(occ[0:int(nsize/2)]))-1
			band_D=band_VBM+1
			band_CBM=band_D+1
			print('band_VBM='+str(band_VBM))
			for ii in range(nkpt):
				E_VBM.append(E[ii*nsize+band_VBM])
				E_VBM.append(E[int(ii*nsize+band_VBM+nsize/2)])
				E_CBM.append(E[ii*nsize+band_CBM])
				E_CBM.append(E[int(ii*nsize+band_CBM+nsize/2)])
				E_D_up.append(E[ii*nsize+band_D])
				E_D_dn.append(E[int(ii*nsize+band_D+nsize/2)])
			VBM = max(E_VBM)
			CBM = min(E_CBM)
			D1 = np.mean(E_D_up)
			D2 = np.mean(E_D_dn)
		if spin == 0: 
			band_VBM=int(np.sum(occ[0:nsize]))-1
			band_D=band_VBM+1
			band_CBM=band_D+1
			print('band_VBM='+str(band_VBM))
			for ii in range(nkpt):
				E_VBM.append(E[ii*nsize+band_VBM])
				E_CBM.append(E[ii*nsize+band_CBM])
				E_D_up.append(E[ii*nsize+band_D])
				E_D_dn.append(E[ii*nsize+band_D])
			VBM = max(E_VBM)
			CBM = min(E_CBM)
			D1 = np.mean(E_D_up)
			D2 = np.mean(E_D_dn)		   
	if charge == 0 :    #0
		E=[]
		occ=[]
		E_VBM=[]
		E_CBM=[]
		E_D_up=[]
		E_D_dn=[]
		f = open(filename,'r')
		data = f.readlines()
		for i,line in enumerate(data):
			s = line.split()
			if s[0][0:5] == "<nks>":
				nkpt = int(s[0][5])
			if s[0] == "<eigenvalues":
				nsize=int(s[1][6:9])
				for line2 in data[i+1:int(i+2+math.ceil(nsize/5))-1]:
					s2 = line2.split()
					for w in s2:
						E.append(float(w))
			if s[0] == "<occupations":
				for line2 in data[i+1:int(i+2+math.ceil(nsize/5))-1]:
					s2 = line2.split()
					for w in s2:
						occ.append(float(w))
		f.close()
		E=27.21138602*np.array(E)
		occ=np.array(occ)
		band_VBM=int(np.sum(occ[0:int(nsize/2)]))-2
		band_D=band_VBM+1
		band_CBM=band_D+1
		print('band_VBM='+str(band_VBM))
		for ii in range(nkpt):
			E_VBM.append(E[ii*nsize+band_VBM])
			E_VBM.append(E[int(ii*nsize+band_VBM+nsize/2)])
			E_CBM.append(E[ii*nsize+band_CBM])
			E_CBM.append(E[int(ii*nsize+band_CBM+nsize/2)])
			E_D_up.append(E[ii*nsize+band_D])
			E_D_dn.append(E[int(ii*nsize+band_D+nsize/2)])
		VBM = max(E_VBM)
		CBM = min(E_CBM)
		D1 = np.mean(E_D_up)
		D2 = np.mean(E_D_dn)
		
	return VBM,CBM,D1,D2


          	
def read_average_potential(filename):
	data = []
	x = []
	with open(filename, 'r') as f:
		lines = f.readlines()
		for line in lines[2:202]:
			x.append(float(line.split()[0]))
			data.append(float(line.split()[1]))	 
	return x,data 


def read_difference_potential(filename):
	data1 = []
	data2 = []
	x = []
	with open(filename, 'r') as f:
		lines = f.readlines()
		for line in lines[207:407]:
			x.append(float(line.split()[0]))
			data1.append(float(line.split()[1]))				
			data2.append(float(line.split()[2]))	 
	return x,data1,data2 

	
def correction(filename3,filename4,copy = False):
	[z3,V3]=read_average_potential(filename3)
	[z4,dV,cV]=read_difference_potential(filename3)
	xnew = np.linspace(np.max([np.min(z3),np.min(z4)])*bohr2A,np.min([np.max(z3),np.max(z4)])*bohr2A,200)
	f3 = interp1d(np.array(z3)*bohr2A,V3,kind = 'cubic')
	f4 = interp1d(np.array(z4)*bohr2A,dV,kind = 'cubic')	
	V3new = f3(xnew)
	dVnew = f4(xnew)
	index_defect = np.argmax(dV)
	if index_defect+np.size(V3new)/2<np.size(V3new):
		index_bulk=int(index_defect+np.size(V3new)/2)
	else:
		index_bulk=int(index_defect-np.size(V3new)/2)
	#align the maximum of V3new with the maximum of dV4new
	i1 = np.argmax(V3new)
	i2 = np.argmax(dVnew)
	V3_copy=np.zeros(np.size(V3new))
	for i in range(np.size(V3new)):
		V3_copy[i]=V3new[i]
	for i in range(np.size(V3new)):
		if i-i2+i1>=0 and i-i2+i1<np.size(V3new):
			V3new[i]=V3_copy[i-i2+i1]
		if i-i2+i1<0:
			V3new[i]=V3_copy[i-i2+i1+np.size(V3new)]
		if i-i2+i1>=np.size(V3new):
			V3new[i]=V3_copy[i-i2+i1-np.size(V3new)]
	Dv = dVnew[index_bulk]-V3new[index_bulk]
	Elat =  0.793486
	Ecorr = Elat - Dv
	if copy == True:
		figure,ax=plt.subplots(facecolor='w',figsize=(10.6299*0.6,8))
		#ax.plot(xnew,V1new)
		ax.plot(xnew,V3new,'--',label=r'V$^{mod}$')
		#ax.plot(xnew,V3_copy,'--',label=r'V$^{mod}$-o')
		ax.plot(xnew,dVnew,label=r'V$_q$-V$_0$')
		ax.plot(xnew,dVnew-V3new,'--',label=r'V$_q$-V$_0$-V$^{mod}$')
		ax.plot([xnew[index_bulk]],[dVnew[index_bulk]-V3new[index_bulk]],'o')
		ax.legend()
		figure.tight_layout()
		figure.savefig(filename4)
	return Dv, Ecorr

	     
		


alpha=[0,0.1,0.2,0.3,0.4]
prefix ='example_MgO'
alpha_name=['probe_a0','probe_oneshot_a01','probe_oneshot_a02','probe_oneshot_a03','probe_oneshot_a04']
#prefix = 'H'
VBM_O=[]
CBM_O=[]
VBM_P=[]
CBM_P=[]
KS_levels_O = []
KS_levels_P = []
corrections = []
for i in alpha_name:
	[VBM,CBM,D1,D2] = grep_ks_level(prefix+'/P_'+i+'.xml',1,1)
	VBM_P.append(VBM)
	CBM_P.append(CBM)
	KS_levels_P.append(D2)
	[VBM,CBM,D1,D2] = grep_ks_level(prefix+'/O_'+i+'.xml',0,1)
	VBM_O.append(VBM)
	CBM_O.append(CBM)
	KS_levels_O.append(D1)
	[Dv,Ecorr] = correction(prefix+'/averagepot_'+i+'.dat',prefix+'/correction_'+i,copy=True)
	corrections.append(Ecorr)


figure,ax=plt.subplots(facecolor='w',figsize=(10.6299*0.6,8))
ax.plot(alpha,KS_levels_O,'-o',label=r'$\mathrm{H}^0$')
KS_levels_P_corr=np.array(KS_levels_P)-2*np.array(corrections)
ax.plot(alpha,KS_levels_P_corr,'-o',label=r'$\mathrm{H}^+$')

ax.plot(alpha,VBM_P,label='VMB')
ax.plot(alpha,CBM_P,label='CMB')
ax.legend()


ax.set_xlabel(r'$\alpha$')
ax.set_ylabel('energy (eV)')
xnew = np.linspace(min(alpha),1,1000)
p1 = np.poly1d(np.polyfit(alpha,KS_levels_O,1))
p2 = np.poly1d(np.polyfit(alpha,KS_levels_P_corr,1))

ynew1 = p1(xnew)
ynew2 = p2(xnew)
ynew = ynew1 - ynew2
for ii in range(np.size(xnew)):
	if ynew[ii]>= 0 and ynew[ii+1]< 0:
		target_alpha = xnew[ii]
		print(xnew[ii])

ax.plot([target_alpha,target_alpha],[min(VBM_P),max(CBM_P)],'--',color='Black')
ax.text(0.1,(min(VBM_P)+max(CBM_P))/2,r'$\alpha_K$='+"{:.3f}".format(target_alpha))
figure.tight_layout()
figure.savefig('koopman-oneshot-probe')
